/**********************************************************************
* FILENAME
*   qmf_short.c
*
* DESCRIPTION
*   Qmf analysis filter providing lowpass- and highpass output
*
* Rev 1.0  Oct 30th 2001 		M. Rauter <rowdy@sbox.tugraz.at>
* Rev 1.1  Nov 29th 2001 		C. Feldbauer
**********************************************************************/

#include "qmf_short.h"


void qmf_short_init( struct_qmf_short *data, short* coeffs, short num_coeff, unsigned blksz)
{
  data->coeffs = coeffs;  // load pointer of qmf coefficients
  data->blksz = blksz;    // set input blocksize
  data->states = (short *)malloc( num_coeff * sizeof(short));  // memory for delay states
  memset( data->states, 0, num_coeff * sizeof(short));  // for first block ->states[i]=0
  data->num_coeff = num_coeff;  // set # of qmf coefficients
  data->write_index = 0;        // set pointer of circular buffer 
}


void qmf_short( struct_qmf_short *data, short *high_out, short *low_out, short *sigbuf)
{
  int i, j, tmp_index, result_even, result_odd;

  for( i=0; i<data->blksz; i++)
  {
    result_even = 0;
    result_odd  = 0; 
    data->states[data->write_index] = sigbuf[i];  // circular buffer
    data->write_index++;							
    data->write_index %= data->num_coeff;         // modulo buffer pointer
		
    /* here the values of the even and the odd branch are calculated */
		
    for( j = data->num_coeff-1, tmp_index = data->write_index;  j>=0; j--)	
    {
      if( j&1) 	// even or odd state?
        result_odd  +=  data->states[(tmp_index++) % data->num_coeff] * data->coeffs[j];
      else
        result_even +=  data->states[(tmp_index++) % data->num_coeff] * data->coeffs[j];      
    }	

    low_out[i]  = (short)( (result_even + result_odd) >> 15);    // LOWPASS OUTPUT
    high_out[i] = (short)( (result_even - result_odd) >> 15);    // HIGHPASS OUTPUT	    
  }	
}
